# Generated by Django 4.2.5 on 2023-10-17 22:25

import _socket
import django.db.models.manager
import django_audit_fields.fields.hostname_modification_field
import edc_screening.model_mixins.screening_model_mixin
import edc_sites.models
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "meta_screening",
            "0058_alter_historicalscreeningpartone_eligibility_datetime_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="icpreferral",
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="subjectrefusal",
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="subjectscreening",
            managers=[
                (
                    "objects",
                    edc_screening.model_mixins.screening_model_mixin.ScreeningManager(),
                ),
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.AlterField(
            model_name="historicalicpreferral",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicalicpreferral",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicalicpreferral",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalicpreferral",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectrefusal",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicalsubjectrefusal",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicalsubjectrefusal",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectrefusal",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="icpreferral",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="icpreferral",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="icpreferral",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="icpreferral",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="subjectrefusal",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="subjectrefusal",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="subjectrefusal",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="subjectrefusal",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="device_created",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device created"),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10, verbose_name="Device modified"),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
                verbose_name="Hostname created",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="hostname_modified",
            field=django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                blank=True,
                help_text="System field. (modified on every save)",
                max_length=50,
                verbose_name="Hostname modified",
            ),
        ),
    ]



from django import forms
from django.utils.translation import gettext_lazy as _
from edc_consent.modelform_mixins import ConsentModelFormMixin
from edc_form_validators import FormValidatorMixin
from edc_sites.forms import SiteModelFormMixin

from ..form_validators import SubjectConsentFormValidator
from ..models import SubjectConsent

help_texts = {
    "identity": (
        _(
            "Use Country ID Number, Passport number, driver's license "
            "number or Country ID receipt number"
        )
    ),
    "witness_name": (
        _(
            "Required only if participant is illiterate. "
            "Format is 'LASTNAME, FIRSTNAME'. "
            "All uppercase separated by a comma."
        )
    ),
    "screening_identifier": _("(read-only)"),
}


widgets = {
    "screening_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
}


class SubjectConsentForm(
    SiteModelFormMixin, ConsentModelFormMixin, FormValidatorMixin, forms.ModelForm
):
    form_validator_cls = SubjectConsentFormValidator

    def validate_gender_of_consent(self):
        return None

    def validate_guardian_and_dob(self):
        return None

    class Meta:
        model = SubjectConsent
        fields = "__all__"
        help_texts = help_texts
        widgets = widgets



# Generated by Django 4.2.11 on 2024-05-15 17:34

import uuid

import _socket
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_crf.model_mixins.crf_status_model_mixin
import edc_model.validators.date
import edc_protocol.validators
import edc_utils.date
import edc_visit_tracking.managers
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("meta_subject", "0185_alter_bloodresultsins_fasting_duration_str_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="healtheconomicsupdate",
            name="singleton_field",
            field=models.CharField(
                editable=False,
                help_text="auto updated for unique constraint",
                max_length=50,
                null=True,
                unique=True,
                verbose_name="subject identifier",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicsupdate",
            name="singleton_field",
            field=models.CharField(
                db_index=True,
                editable=False,
                help_text="auto updated for unique constraint",
                max_length=50,
                null=True,
                verbose_name="subject identifier",
            ),
        ),
    ]



# Generated by Django 3.2.4 on 2021-06-28 20:31

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0054_auto_20210628_2314"),
    ]

    operations = [
        migrations.AddField(
            model_name="bloodresultsrft",
            name="egfr_abnormal",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                null=True,
                verbose_name="abnormal",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="egfr_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="egfr_units",
            field=models.CharField(
                blank=True,
                default="mL/min/1.73 m2",
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="egfr_abnormal",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                null=True,
                verbose_name="abnormal",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="egfr_reportable",
            field=models.CharField(
                blank=True,
                choices=[
                    ("N/A", "Not applicable"),
                    ("3", "Yes, grade 3"),
                    ("4", "Yes, grade 4"),
                    ("No", "Not reportable"),
                    ("Already reported", "Already reported"),
                    ("present_at_baseline", "Present at baseline"),
                ],
                max_length=25,
                null=True,
                verbose_name="reportable",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="egfr_units",
            field=models.CharField(
                blank=True,
                default="mL/min/1.73 m2",
                max_length=15,
                null=True,
                verbose_name="units",
            ),
        ),
    ]



# Generated by Django 5.1.2 on 2024-10-10 04:23

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_pharmacy", "0023_remove_rx_edc_pharmac_modifie_986021_idx_and_more"),
        ("meta_pharmacy", "0006_lotnumber_label"),
    ]

    operations = [
        migrations.AddField(
            model_name="lotnumber",
            name="medication",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="edc_pharmacy.medication",
            ),
        ),
    ]



from edc_constants.constants import BLACK, NO, NOT_APPLICABLE, OTHER, PENDING, YES

ETHNICITY = ((BLACK, "Black"), (OTHER, "Other"))

REFUSAL_REASONS = (
    ("dont_have_time", "I don't have time"),
    ("must_consult_spouse", "I need to consult my spouse"),
    ("dont_want_blood_drawn", "I don't want to have the blood drawn"),
    ("dont_want_to_join", "I don't want to take part"),
    ("need_to_think_about_it", "I haven't had a chance to think about it"),
    (OTHER, "Other, please specify"),
)


YES_NO_NOT_ELIGIBLE = (
    (YES, YES),
    (NO, NO),
    (
        NOT_APPLICABLE,
        "Not applicable, subject is not eligible based on the criteria above",
    ),
)

YES_PENDING_NA = (
    (YES, YES),
    (PENDING, "Decision pending"),
    (NOT_APPLICABLE, "Not applicable"),
)



# Generated by Django 3.2.11 on 2022-03-24 00:26

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0102_auto_20220324_0304"),
    ]

    operations = [
        migrations.AlterField(
            model_name="delivery",
            name="info_not_available_reason",
            field=models.TextField(
                blank=True,
                null=True,
                verbose_name="If the report was not available, please explain?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldelivery",
            name="info_not_available_reason",
            field=models.TextField(
                blank=True,
                null=True,
                verbose_name="If the report was not available, please explain?",
            ),
        ),
    ]



# Generated by Django 4.2.10 on 2024-03-07 23:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0055_alter_endofstudy_site_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="endofstudy",
            name="clinical_withdrawal_reason",
            field=models.CharField(
                choices=[
                    ("kidney_disease", "Development of chronic kidney disease"),
                    ("liver_disease", "Development of chronic liver disease"),
                    (
                        "intercurrent_illness",
                        "Intercurrent illness which prevents further treatment",
                    ),
                    ("investigator_decision", "Investigator decision (specify below)"),
                    (
                        "OTHER",
                        "Other condition that justifies the discontinuation of treatment in the clinician's opinion (specify below)",
                    ),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="If the patient was withdrawn on CLINICAL grounds, please specify PRIMARY reason",
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="clinical_withdrawal_reason",
            field=models.CharField(
                choices=[
                    ("kidney_disease", "Development of chronic kidney disease"),
                    ("liver_disease", "Development of chronic liver disease"),
                    (
                        "intercurrent_illness",
                        "Intercurrent illness which prevents further treatment",
                    ),
                    ("investigator_decision", "Investigator decision (specify below)"),
                    (
                        "OTHER",
                        "Other condition that justifies the discontinuation of treatment in the clinician's opinion (specify below)",
                    ),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="If the patient was withdrawn on CLINICAL grounds, please specify PRIMARY reason",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="reason",
            field=models.CharField(
                choices=[
                    ("pregnancy", "Pregnancy"),
                    ("sae", "Participant is experiencing a serious adverse event"),
                    (
                        "clinician",
                        "Other condition that justifies discontinuation of treatment in the clinician's opinion (specify below)",
                    ),
                    ("investigator", " Investigator decision"),
                    ("patient", "Patient decision"),
                    ("OTHER", "Other reason (specify below)"),
                ],
                max_length=25,
                verbose_name="Reason for stopping study medication",
            ),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="reason",
            field=models.CharField(
                choices=[
                    ("pregnancy", "Pregnancy"),
                    ("sae", "Participant is experiencing a serious adverse event"),
                    (
                        "clinician",
                        "Other condition that justifies discontinuation of treatment in the clinician's opinion (specify below)",
                    ),
                    ("investigator", " Investigator decision"),
                    ("patient", "Patient decision"),
                    ("OTHER", "Other reason (specify below)"),
                ],
                max_length=25,
                verbose_name="Reason for stopping study medication",
            ),
        ),
    ]



from edc_adverse_event.model_mixins import AeSusarModelMixin
from edc_model.models import BaseUuidModel


class AeSusar(AeSusarModelMixin, BaseUuidModel):
    class Meta(AeSusarModelMixin.Meta):
        pass



# Generated by Django 5.1 on 2024-10-07 22:02

import edc_utils.date
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0049_auto_20240911_0327"),
    ]

    operations = [
        migrations.AlterField(
            model_name="endpoints",
            name="created",
            field=models.DateTimeField(default=edc_utils.date.get_utcnow),
        ),
    ]



from typing import Tuple

from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import meta_screening_admin
from ..forms import (
    ScreeningPartTwoForm,
    calculated_fields,
    part_one_fields,
    part_three_fields,
)
from ..models import ScreeningPartTwo
from .fieldsets import (
    get_p3_screening_appt_update_fields,
    get_part_one_fieldset,
    get_part_three_fieldset,
    get_part_two_fieldset,
)
from .subject_screening_admin import SubjectScreeningAdmin


def get_fieldsets() -> Tuple[
    Tuple[str, dict],
    Tuple[str, dict],
    Tuple[str, dict],
    Tuple[str, dict],
    Tuple[str, dict],
]:
    return (
        get_part_one_fieldset(collapse=True),
        get_part_two_fieldset(),
        get_p3_screening_appt_update_fields(),
        get_part_three_fieldset(collapse=True),
        audit_fieldset_tuple,
    )


@admin.register(ScreeningPartTwo, site=meta_screening_admin)
class ScreeningPartTwoAdmin(SiteModelAdminMixin, SubjectScreeningAdmin):

    form = ScreeningPartTwoForm

    fieldsets = get_fieldsets()

    readonly_fields: Tuple[str, ...] = (
        *part_one_fields,
        *part_three_fields,
        *calculated_fields,
    )



from django.core.exceptions import ImproperlyConfigured


def confirm_meta_version(phase):
    response = input(f"This is meta phase {phase}. Type 2 or 3 to continue: ")
    if response not in ["2", "3"]:
        print("settings load has been halted.")
        raise ImproperlyConfigured("Check settings.META_PHASE")
    return int(response)



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.fieldset import crf_status_fieldset
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_subject_admin
from ..forms import PhysicalExamForm
from ..models import PhysicalExam
from .fields import get_blood_pressure_fields
from .modeladmin import CrfModelAdminMixin


def get_other_vitals_fieldset():
    return (
        "Part 2: Other vitals",
        {"fields": ["temperature", "waist_circumference"]},
    )


@admin.register(PhysicalExam, site=meta_subject_admin)
class PhysicalExamAdmin(CrfModelAdminMixin, SimpleHistoryAdmin):
    form = PhysicalExamForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Part 1: BP and Heart",
            {
                "fields": (
                    *get_blood_pressure_fields(),
                    "heart_rate",
                    "irregular_heartbeat",
                    "irregular_heartbeat_description",
                    "respiratory_rate",
                )
            },
        ),
        get_other_vitals_fieldset(),
        (
            "Part 3: Signs",
            {
                "fields": (
                    "jaundice",
                    "peripheral_oedema",
                    "abdominal_tenderness",
                    "abdominal_tenderness_description",
                    "enlarged_liver",
                )
            },
        ),
        crf_status_fieldset,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "abdominal_tenderness": admin.VERTICAL,
        "enlarged_liver": admin.VERTICAL,
        "irregular_heartbeat": admin.VERTICAL,
        "jaundice": admin.VERTICAL,
        "peripheral_oedema": admin.VERTICAL,
        "severe_htn": admin.VERTICAL,
    }



import pandas as pd
from edc_analytics.custom_tables import AgeTable, GenderTable
from edc_analytics.table import Table


class GlucoseTable(Table):
    """
    gender
    age
    fasting
    fbg mean
    ogtt mean
    fbg + ogtt count
    """

    def __init__(self, main_df: pd.DataFrame = None):
        super().__init__(colname="", main_df=main_df, title="Glucose (enrolled)")

    def build_table_df(self) -> None:
        df = self.main_df
        s = df.groupby("subject_identifier")[["age_in_years", "gender"]].value_counts()
        df_tmp = s.to_frame()
        df_tmp = df_tmp.reset_index()
        gender_tbl = GenderTable(main_df=df_tmp).table_df
        age_tbl = AgeTable(main_df=df_tmp).table_df
        return pd.concat([gender_tbl, age_tbl])



from edc_model.models import BaseUuidModel
from edc_qol.model_mixins import Eq5d3lModelMixin

from ..model_mixins import CrfModelMixin


class Eq5d3l(Eq5d3lModelMixin, CrfModelMixin, BaseUuidModel):
    class Meta(Eq5d3lModelMixin.Meta, CrfModelMixin.Meta, BaseUuidModel.Meta):
        pass



from django.core.exceptions import ObjectDoesNotExist
from edc_data_manager.handlers import QueryRuleHandler

from meta_screening.models import SubjectScreening


class BaselineHbA1cRuleHandler(QueryRuleHandler):
    name = "baseline_hba1c"
    display_name = "Baseline HbA1c"
    model_name = "meta_subject.bloodresultshba1c"

    def run(self):
        try:
            SubjectScreening.objects.get(
                subject_identifier=self.visit_obj.subject_identifier,
                hba1c_value__isnull=False,
            )
        except ObjectDoesNotExist:
            if self.resolved:
                self.data_query = self.get_or_create_data_query(get_only=True)
                if self.data_query:
                    self.resolve_existing_data_query()
            else:
                self.data_query = self.get_or_create_data_query()
                if self.data_query.site_resolved:
                    self.reopen_existing_data_query()



from django.apps import apps as django_apps
from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from edc_appointment.models import Appointment
from edc_model_admin.dashboard import ModelAdminDashboardMixin
from edc_model_admin.list_filters import ReportDateListFilter
from edc_model_admin.mixins import TemplatesModelAdminMixin
from edc_qareports.modeladmin_mixins import QaReportModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin
from edc_visit_schedule.admin import ScheduleStatusListFilter
from edc_visit_schedule.constants import DAY1

from ...admin_site import meta_reports_admin
from ...models import ImpSubstitutions


@admin.register(ImpSubstitutions, site=meta_reports_admin)
class ImpSubstitutionsAdmin(
    QaReportModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminDashboardMixin,
    TemplatesModelAdminMixin,
    admin.ModelAdmin,
):
    ordering = ["site", "subject_identifier"]
    list_display = [
        "dashboard",
        "render_button",
        "subject",
        "sid",
        "dispensed_sid",
        "report_date",
        "arm_match",
        "allocated_date",
        "user_created",
        "user_modified",
        "modified",
    ]

    list_filter = [
        "arm_match",
        ScheduleStatusListFilter,
        ReportDateListFilter,
        "allocated_datetime",
    ]

    search_fields = ["subject_identifier", "sid", "dispensed_sid"]

    def dashboard(self, obj=None, label=None) -> str:
        kwargs = self.get_subject_dashboard_url_kwargs(obj)
        try:
            kwargs.update(
                appointment=str(
                    Appointment.objects.get(
                        subject_identifier=obj.subject_identifier,
                        visit_code=DAY1,
                        visit_code_sequence=0,
                    ).id
                )
            )
        except ObjectDoesNotExist:
            pass
        url = reverse(self.get_subject_dashboard_url_name(obj=obj), kwargs=kwargs)
        context = dict(title=_("Go to subject's dashboard@1000"), url=url, label=label)
        return render_to_string("dashboard_button.html", context=context)

    @admin.display(description="Subject", ordering="subject_identifier")
    def subject(self, obj):
        return obj.subject_identifier

    @admin.display(description="Allocated", ordering="allocated_datetime")
    def allocated_date(self, obj=None):
        return obj.allocated_datetime.date() if obj.allocated_datetime else None

    @admin.display(description="Update")
    def render_button(self, obj=None):
        crf_model_cls = django_apps.get_model("meta_pharmacy", "substitutions")
        url = reverse(
            f"meta_pharmacy_admin:{crf_model_cls._meta.label_lower.replace('.', '_')}_change",
            args=(obj.original_id,),
        )
        url = (
            f"{url}?next={self.admin_site.name}:"
            f"{self.model._meta.label_lower.replace('.', '_')}_changelist"
        )
        title = _(f"View {crf_model_cls._meta.verbose_name}")
        label = _("View")
        crf_button = render_to_string(
            "edc_qareports/columns/change_button.html",
            context=dict(title=title, url=url, label=label),
        )
        return crf_button

    @admin.display(description="Report date", ordering="report_datetime")
    def report_date(self, obj) -> str | None:
        if obj.report_datetime:
            return obj.report_datetime.date()
        return None



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.fieldset import crf_status_fieldset
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ...admin_site import meta_subject_admin
from ...forms import DmEndpointForm
from ...models import DmEndpoint
from ..modeladmin import CrfModelAdminMixin


@admin.register(DmEndpoint, site=meta_subject_admin)
class DmEndpointAdmin(
    CrfModelAdminMixin,
    FormLabelModelAdminMixin,
    SimpleHistoryAdmin,
):
    form = DmEndpointForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Endpoint reached",
            {
                "fields": ("endpoint_reached", "comments"),
            },
        ),
        crf_status_fieldset,
        audit_fieldset_tuple,
    )

    list_filter = ["endpoint_reached"]

    radio_fields = {"endpoint_reached": admin.VERTICAL}



from datetime import datetime
from random import sample
from secrets import choice
from string import ascii_uppercase
from zoneinfo import ZoneInfo

from dateutil.relativedelta import relativedelta
from django.conf import settings
from django.contrib.sites.models import Site
from edc_constants.constants import (
    BLACK,
    FEMALE,
    NO,
    NOT_APPLICABLE,
    RANDOM_SAMPLING,
    YES,
)
from edc_reportable.units import MICROMOLES_PER_LITER, MILLIMOLES_PER_LITER
from faker import Faker

from meta_edc.meta_version import get_meta_version
from meta_screening.forms import part_one_fields, part_three_fields, part_two_fields

fake = Faker()
now = datetime(2019, 5, 1).astimezone(ZoneInfo("UTC"))
tomorrow = now + relativedelta(days=1)


def get_part_one_eligible_options():
    options = dict(
        age_in_years=25,
        art_six_months=YES,
        continue_part_two=YES,
        ethnicity=BLACK,
        gender=FEMALE,
        hiv_pos=YES,
        hospital_identifier="".join(map(str, sample(range(0, 10), 10))),
        initials=f"{choice(ascii_uppercase)}{choice(ascii_uppercase)}",
        lives_nearby=YES,
        on_rx_stable=YES,
        pregnant=NOT_APPLICABLE,
        report_datetime=now,
        screening_consent=YES,
        selection_method=RANDOM_SAMPLING,
        staying_nearby_12=YES,
        vl_undetectable=YES,
        meta_phase_two=NO,
        site=Site.objects.get(id=settings.SITE_ID),
    )
    if fld := [f for f in part_one_fields if f not in options]:
        raise TypeError(
            f"Missing part one fields for meta phase {get_meta_version()}. Got {fld}."
        )
    return options


def get_part_two_eligible_options(report_datetime: datetime = None) -> dict:
    options = dict(
        acute_condition=NO,
        acute_metabolic_acidosis=NO,
        advised_to_fast=YES,
        alcoholism=NO,
        already_fasted=NO,
        appt_datetime=report_datetime or (now + relativedelta(days=10)),
        congestive_heart_failure=NO,
        liver_disease=NO,
        metformin_sensitivity=NO,
        part_two_report_datetime=report_datetime or now,
        renal_function_condition=NO,
        tissue_hypoxia_condition=NO,
        has_dm=NO,
        on_dm_medication=NO,
        agree_to_p3=YES,
        p3_ltfu=NOT_APPLICABLE,
        p3_ltfu_date=None,
        p3_ltfu_comment=None,
        contact_number=None,
    )
    if fld := [f for f in part_two_fields if f not in options]:
        raise TypeError(
            f"Missing part two fields for meta phase {get_meta_version()}. Got {fld}."
        )
    return options


def get_part_three_eligible_options(report_datetime: datetime = None):
    options = dict(
        creatinine_performed=YES,
        creatinine_units=MICROMOLES_PER_LITER,
        creatinine_value=50,
        dia_blood_pressure=65,
        dia_blood_pressure_one=65,
        dia_blood_pressure_two=65,
        fasting=YES,
        fasting_duration_str="8h",
        hba1c_performed=YES,
        hba1c_datetime=report_datetime or now,
        hba1c_value=7.0,
        height=110,
        fbg_datetime=tomorrow,
        fbg_units=MILLIMOLES_PER_LITER,
        fbg_value=6.9,
        ogtt_base_datetime=tomorrow + relativedelta(minutes=5),
        ogtt_datetime=tomorrow + relativedelta(hours=2),
        ogtt_units=MILLIMOLES_PER_LITER,
        ogtt_value=7.8,
        repeat_glucose_opinion=NO,
        ogtt2_performed=NOT_APPLICABLE,
        part_three_report_datetime=report_datetime or now,
        reasons_unsuitable=None,
        sys_blood_pressure=120,
        sys_blood_pressure_one=120,
        sys_blood_pressure_two=120,
        unsuitable_agreed=NOT_APPLICABLE,
        unsuitable_for_study=NO,
        urine_bhcg_date=None,
        urine_bhcg_performed=NOT_APPLICABLE,
        urine_bhcg_value=NOT_APPLICABLE,
        waist_circumference=130,
        weight=65,
        severe_htn=NO,
    )
    if fld := [f for f in part_three_fields if f not in options]:
        raise TypeError(
            f"Missing part three fields for meta phase {get_meta_version()}. Got {fld}."
        )
    return options



# Generated by Django 5.0.7 on 2024-08-12 14:20

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0009_alter_endpoints_options"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="patienthistorymissingbaselinecd4",
            options={
                "default_permissions": ("view", "export", "viewallsites"),
                "managed": False,
                "verbose_name": "Patient History: Missing Baseline Cd4",
                "verbose_name_plural": "Missing Baseline Cd4",
            },
        ),
        migrations.AlterModelOptions(
            name="unattendedthreeinrow",
            options={
                "default_permissions": ("view", "export", "viewallsites"),
                "managed": False,
                "verbose_name": "R100: Unattended appointments: Three in a row",
                "verbose_name_plural": "R100: Unattended appointments: Three in a row",
            },
        ),
        migrations.AlterModelOptions(
            name="unattendedthreeinrow2",
            options={
                "default_permissions": ("view", "export", "viewallsites"),
                "managed": False,
                "verbose_name": "R110: Unattended appointments: Three in a row (with missed)",
                "verbose_name_plural": "R110: Unattended appointments: Three in a row (with missed)",
            },
        ),
        migrations.AlterModelOptions(
            name="unattendedtwoinrow",
            options={
                "default_permissions": ("view", "export", "viewallsites"),
                "managed": False,
                "verbose_name": "R120: Unattended appointments: Two in a row",
                "verbose_name_plural": "R120: Unattended appointments: Two in a row",
            },
        ),
    ]



# Generated by Django 4.2.11 on 2024-05-15 23:47

from django.db import migrations
from edc_constants.constants import YES


def update_fasting_duration(apps, schema_editor):
    model_cls = apps.get_model("meta_subject.glucosefbg")
    model_cls.objects.filter(fasting_duration_estimated=YES).update(fasting_duration_str="8h")


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0198_alter_glucosefbg_fasting_duration_estimated_and_more"),
    ]

    operations = [migrations.RunPython(update_fasting_duration)]



from edc_protocol_incident.forms import ProtocolIncidentForm as Base

from ..models import ProtocolIncident


class ProtocolIncidentForm(Base):
    class Meta(Base.Meta):
        model = ProtocolIncident
        fields = "__all__"



from typing import Tuple

from django.contrib import admin
from django.utils.html import format_html
from django_audit_fields import audit_fieldset_tuple
from edc_action_item import ActionItemModelAdminMixin, action_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import meta_prn_admin
from ..forms import OffScheduleForm
from ..models import EndOfStudy, OffSchedule


@admin.register(OffSchedule, site=meta_prn_admin)
class OffScheduleAdmin(
    SiteModelAdminMixin,
    ActionItemModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    SimpleHistoryAdmin,
):
    form = OffScheduleForm

    additional_instructions = format_html(
        '<span style="color:orange;font-weight:bold">Note:</span> Detailed '
        "information about study termination will be asked for on the "
        f"<b>{EndOfStudy._meta.verbose_name}</b> form"
    )

    fieldsets = (
        (None, {"fields": ("subject_identifier", "offschedule_datetime")}),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    def get_list_display(self, request) -> Tuple[str, ...]:
        list_display = super().get_list_display(request)
        custom_fields = ("subject_identifier", "dashboard", "offschedule_datetime")
        return custom_fields + tuple(f for f in list_display if f not in custom_fields)

    def get_list_filter(self, request) -> Tuple[str, ...]:
        list_filter = super().get_list_filter(request)
        custom_fields = ("offschedule_datetime",)
        return custom_fields + tuple(f for f in list_filter if f not in custom_fields)



# Generated by Django 3.2.4 on 2021-06-23 23:25

import django.db.models.deletion
import edc_model.validators.date
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_action_item", "0028_auto_20210203_0706"),
        ("meta_ae", "0004_auto_20191114_0821"),
    ]

    operations = [
        migrations.AddField(
            model_name="deathreport",
            name="death_date",
            field=models.DateField(
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date of Death",
            ),
        ),
        migrations.AddField(
            model_name="historicaldeathreport",
            name="death_date",
            field=models.DateField(
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date of Death",
            ),
        ),
        migrations.AlterField(
            model_name="aefollowup",
            name="action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="aefollowup",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="aefollowup",
            name="parent_action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="aefollowup",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="aefollowup",
            name="related_action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="aeinitial",
            name="action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="aeinitial",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="aeinitial",
            name="parent_action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="aeinitial",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="aeinitial",
            name="related_action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="aesusar",
            name="action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="aesusar",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="aesusar",
            name="parent_action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="aesusar",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="aesusar",
            name="related_action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="aetmg",
            name="action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="aetmg",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="aetmg",
            name="parent_action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="aetmg",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="aetmg",
            name="related_action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="deathreport",
            name="action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="deathreport",
            name="death_as_inpatient",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=5,
                null=True,
                verbose_name="Death as inpatient",
            ),
        ),
        migrations.AlterField(
            model_name="deathreport",
            name="death_datetime",
            field=models.DateTimeField(
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date and Time of Death",
            ),
        ),
        migrations.AlterField(
            model_name="deathreport",
            name="narrative",
            field=models.TextField(null=True, verbose_name="Narrative"),
        ),
        migrations.AlterField(
            model_name="deathreport",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="deathreport",
            name="parent_action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="deathreport",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="deathreport",
            name="related_action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="deathreporttmg",
            name="action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="deathreporttmg",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="deathreporttmg",
            name="parent_action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="deathreporttmg",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="deathreporttmg",
            name="related_action_item",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_action_item.actionitem",
            ),
        ),
        migrations.AlterField(
            model_name="historicalaefollowup",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalaefollowup",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalaeinitial",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalaeinitial",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalaesusar",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalaesusar",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalaetmg",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalaetmg",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicaldeathreport",
            name="death_as_inpatient",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=5,
                null=True,
                verbose_name="Death as inpatient",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldeathreport",
            name="death_datetime",
            field=models.DateTimeField(
                null=True,
                validators=[edc_model.validators.date.datetime_not_future],
                verbose_name="Date and Time of Death",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldeathreport",
            name="narrative",
            field=models.TextField(null=True, verbose_name="Narrative"),
        ),
        migrations.AlterField(
            model_name="historicaldeathreport",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicaldeathreport",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicaldeathreporttmg",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicaldeathreporttmg",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicaldeathreporttmgsecond",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicaldeathreporttmgsecond",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
                null=True,
            ),
        ),
    ]



# Generated by Django 5.0.8 on 2024-08-21 22:20

import django_db_views.migration_functions
import django_db_views.operations
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0025_auto_20240822_0115"),
    ]

    operations = [
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "# noqa\nselect *, uuid() as id, now() as created, 'meta_reports.glucose_summary_view' as report_model\nfrom (\n    select subject_identifier, fbg_value, fbg_datetime, null as 'ogtt_value', null as 'ogtt_datetime', fbg.site_id  \n    from meta_subject_glucosefbg as fbg \n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    UNION\n    select subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime, fbg.site_id\n    from meta_subject_glucose as fbg \n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id) as A\norder by subject_identifier, fbg_datetime",
                "glucose_summary_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "# noqa\nselect subject_identifier, fbg_value, fbg_datetime, null as 'ogtt_value', null as 'ogtt_datetime',\nfbg.site_id, 'glucose_summary_view' as report_model \nfrom meta_subject_glucosefbg as fbg \nleft join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\nUNION\nselect subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,\nfbg.site_id, 'glucose_summary_view' as report_model\nfrom meta_subject_glucose as fbg \nleft join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\norder by subject_identifier, fbg_datetime",
                "glucose_summary_view",
                engine="django.db.backends.mysql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "# noqa\nselect *, get_random_uuid() as id, now() as created, 'meta_reports.glucose_summary_view' as report_model\nfrom (\n    select subject_identifier, fbg_value, fbg_datetime, null as ogtt_value, null as ogtt_datetime, fbg.site_id\n    from meta_subject_glucosefbg as fbg \n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    UNION\n    select subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime, fbg.site_id\n    from meta_subject_glucose as fbg \n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id) as A\norder by subject_identifier, fbg_datetime",
                "glucose_summary_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "# noqa\nselect subject_identifier, fbg_value, fbg_datetime, null as ogtt_value, null as ogtt_datetime,\nfbg.site_id, 'glucose_summary_view' as report_model\nfrom meta_subject_glucosefbg as fbg \nleft join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\nUNION\nselect subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,\nfbg.site_id, 'glucose_summary_view' as report_model\nfrom meta_subject_glucose as fbg \nleft join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\norder by subject_identifier, fbg_datetime",
                "glucose_summary_view",
                engine="django.db.backends.postgresql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "# noqa\nSELECT *, lower(\n    hex(randomblob(4)) || '-' || hex(randomblob(2)) || '-' || '4' ||\n    substr(hex( randomblob(2)), 2) || '-' ||\n    substr('AB89', 1 + (abs(random()) % 4) , 1)  ||\n    substr(hex(randomblob(2)), 2) || '-' ||\n    hex(randomblob(6))\n  ) as id, datetime() as `created`, 'meta_reports.glucose_summary_view' as report_model\nfrom (\n    select subject_identifier, fbg_value, fbg_datetime, null as ogtt_value, null as ogtt_datetime,\n    fbg.site_id, 'glucose_summary_view' as report_model\n    from meta_subject_glucosefbg as fbg \n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\n    UNION\n    select subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,\n    fbg.site_id, 'glucose_summary_view' as report_model\n    from meta_subject_glucose as fbg \n    left join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id) as A\norder by subject_identifier, fbg_datetime",
                "glucose_summary_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "# noqa\nselect subject_identifier, fbg_value, fbg_datetime, null as ogtt_value, null as ogtt_datetime,\nfbg.site_id, 'glucose_summary_view' as report_model\nfrom meta_subject_glucosefbg as fbg \nleft join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\nUNION\nselect subject_identifier, fbg_value, fbg_datetime, ogtt_value, ogtt_datetime,\nfbg.site_id, 'glucose_summary_view' as report_model\nfrom meta_subject_glucose as fbg \nleft join meta_subject_subjectvisit as v on v.id=fbg.subject_visit_id\norder by subject_identifier, fbg_datetime",
                "glucose_summary_view",
                engine="django.db.backends.sqlite3",
            ),
            atomic=False,
        ),
    ]



# Generated by Django 4.0.5 on 2022-06-24 09:34

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0111_alter_followupvitals_severe_htn_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalsubjectvisit",
            name="document_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this document",
            ),
        ),
        migrations.AddField(
            model_name="subjectvisit",
            name="document_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this document",
            ),
        ),
    ]



from django.db import models
from edc_constants.choices import YES_NO, YES_NO_RETIRED
from edc_constants.constants import QUESTION_RETIRED
from edc_model.models import BaseUuidModel
from edc_model_fields.fields import OtherCharField

from ...model_mixins import CrfModelMixin


class DmEndpoint(CrfModelMixin, BaseUuidModel):
    """A user model to enrolled patient onto the DM Referral
    Schedule.
    """

    endpoint_reached = models.CharField(
        verbose_name="Was the patient referred because the diabetes endpoint was reached?",
        max_length=15,
        choices=YES_NO,
        null=True,
        blank=False,
        help_text="If YES, the EDC will check for the patient on the Endpoints report.",
    )

    # retired
    dx_date = models.DateField(verbose_name="Date endpoint reached", null=True)

    # retired
    dx_initiated_by = models.CharField(
        verbose_name="What initiated the diagnosis",
        max_length=25,
        default=QUESTION_RETIRED,
    )

    # retired
    dx_initiated_by_other = OtherCharField(null=True, blank=True)

    # retired
    dx_tmg = models.CharField(
        verbose_name="Was this case discussed with the TMG?",
        max_length=25,
        choices=YES_NO_RETIRED,
        default=QUESTION_RETIRED,
    )

    # retired
    dx_tmg_date = models.DateField(
        verbose_name="If YES, provide the date of the TMG discussion", null=True, blank=True
    )

    # retired
    dx_no_tmg_reason = models.TextField(
        verbose_name="If NO, please explain why this case was not discussed with the TMG",
        null=True,
        blank=True,
    )

    comments = models.TextField(verbose_name="Any other comments", null=True, blank=True)

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Diabetes endpoint"
        verbose_name_plural = "Diabetes endpoint"



# Generated by Django 3.0.6 on 2020-06-14 16:34

from django.db import migrations
from django.db.migrations import RunPython
from edc_action_item.utils import update_action_identifier

from meta_edc.meta_version import PHASE_TWO, get_meta_version
from meta_subject.action_items import FollowupExaminationAction


def update_followup_examination_action_identifier(apps, schema_editor):
    if get_meta_version() == PHASE_TWO:
        update_action_identifier(
            model="meta_subject.followupexamination",
            action_cls=FollowupExaminationAction,
            apps=apps,
        )


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0049_auto_20200613_2041"),
    ]

    operations = [
        RunPython(
            update_followup_examination_action_identifier,
            hints={"target_db": "default"},
        ),
    ]



from django import forms

from ..models import AeLocalReview
from .modelform_mixins import AeReviewModelFormMixin


class AeLocalReviewForm(AeReviewModelFormMixin, forms.ModelForm):
    class Meta(AeReviewModelFormMixin.Meta):
        model = AeLocalReview
        fields = "__all__"



# Generated by Django 2.2.6 on 2019-10-31 04:45

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("meta_screening", "0007_auto_20191024_1000")]

    operations = [
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="pregnant",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not Applicable: e.g. mentruating, post menopausal, not female",
                    ),
                ],
                max_length=15,
                verbose_name="Is the patient pregnant?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="urine_bhcg_performed",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not Applicable: e.g. mentruating, post menopausal, not female",
                    ),
                ],
                default="N/A",
                help_text="(Pregnancy test)",
                max_length=15,
                verbose_name="Was a Urine or serum βhCG performed?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="pregnant",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not Applicable: e.g. mentruating, post menopausal, not female",
                    ),
                ],
                max_length=15,
                verbose_name="Is the patient pregnant?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="urine_bhcg_performed",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not Applicable: e.g. mentruating, post menopausal, not female",
                    ),
                ],
                default="N/A",
                help_text="(Pregnancy test)",
                max_length=15,
                verbose_name="Was a Urine or serum βhCG performed?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="pregnant",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not Applicable: e.g. mentruating, post menopausal, not female",
                    ),
                ],
                max_length=15,
                verbose_name="Is the patient pregnant?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="urine_bhcg_performed",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not Applicable: e.g. mentruating, post menopausal, not female",
                    ),
                ],
                default="N/A",
                help_text="(Pregnancy test)",
                max_length=15,
                verbose_name="Was a Urine or serum βhCG performed?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="pregnant",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not Applicable: e.g. mentruating, post menopausal, not female",
                    ),
                ],
                max_length=15,
                verbose_name="Is the patient pregnant?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="urine_bhcg_performed",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not Applicable: e.g. mentruating, post menopausal, not female",
                    ),
                ],
                default="N/A",
                help_text="(Pregnancy test)",
                max_length=15,
                verbose_name="Was a Urine or serum βhCG performed?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="pregnant",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not Applicable: e.g. mentruating, post menopausal, not female",
                    ),
                ],
                max_length=15,
                verbose_name="Is the patient pregnant?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="urine_bhcg_performed",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    (
                        "N/A",
                        "Not Applicable: e.g. mentruating, post menopausal, not female",
                    ),
                ],
                default="N/A",
                help_text="(Pregnancy test)",
                max_length=15,
                verbose_name="Was a Urine or serum βhCG performed?",
            ),
        ),
    ]
